@@ -17,7 +17,7 @@
 //  <http://www.gnu.org/licenses/>.
 //
 // ***************************************************************************
-// |docname| - Core code for the CodeChat client
+// |docname| - Core code for the CodeChat Client
 // ***************************************************************************
 // Constants
 // =========
@@ -47,7 +47,7 @@ let percent_regex = new RegExp(
 
 // Core client
 // ===========
-// Given an ID to use, run the CodeChat client.
+// Given an ID to use, run the CodeChat Client.
 function run_client(id, ws_address)
 {
     // Get commonly-used nodes in the DOM.
@@ -75,29 +75,29 @@ function run_client(id, ws_address)
     splitMe.init();
     set_splitter_percent(splitter_size[errors_or_warnings]);
 
-    // Create a websocket to communicate with the CodeChat server.
+    // Create a websocket to communicate with the CodeChat Server.
     let ws = new ReconnectingWebSocket(ws_address);
 
     // Identify this client on connection.
     ws.onopen = () => {
-        console.log(`CodeChat client: websocket to CodeChat server open. Sending ID of ${id}.`);
+        console.log(`CodeChat Client: websocket to CodeChat Server open. Sending ID of ${id}.`);
         ws.send(JSON.stringify(id))
     };
 
     // Provide logging to help track down errors.
     ws.onerror = event => {
-        console.error(`CodeChat client: websocket error ${event}.`);
+        console.error(`CodeChat Client: websocket error ${event}.`);
     };
 
     ws.onclose = event => {
-        console.log(`CodeChat client: websocket closed by event ${event}. This should only happen on shutdown.`);
+        console.log(`CodeChat Client: websocket closed by event ${event}. This should only happen on shutdown.`);
     }
 
     // Handle messages.
     ws.onmessage = event => {
         result = JSON.parse(event.data);
         if (result.get_result_type === GetResultType.url) {
-            console.log(`CodeChat client: URL ${result.text} received; loading...`);
+            console.log(`CodeChat Client: URL ${result.text} received; loading...`);
             // Save and restore scroll location through the content update, if we can.
             let [scrollX, scrollY] = getScroll();
             // See ideas in https://stackoverflow.com/a/16822995. Works for same-domain only.
@@ -105,7 +105,7 @@ function run_client(id, ws_address)
                 // Only run this once, not every time the user navigates in the browser.
                 outputElement.onload = undefined;
                 if (is_user_navigation) {
-                    console.log("CodeChat client: TODO: User navigation.");
+                    console.log("CodeChat Client: TODO: User navigation.");
                 } else {
                     status_message.innerHTML = "Build complete.";
                     build_progress.style.display = "none";
@@ -122,7 +122,7 @@ function run_client(id, ws_address)
                     // #.   Before the load finishes, another request comes. The x, y coordinates are saved as 0.
                     // #.   The first load finishes, but scrolls to 0, 0; the second load finish does the same.
 
-                    console.log("CodeChat client: load complete.");
+                    console.log("CodeChat Client: load complete.");
                 }
             };
 
@@ -137,7 +137,7 @@ function run_client(id, ws_address)
 
         } else if (result.get_result_type === GetResultType.build) {
             // Omit this logging, since it's usually obvious from the lines below.
-            ///console.log("CodeChat client: received build output.");
+            ///console.log("CodeChat Client: received build output.");
             if (clear_output) {
                 // This is the start of a new build.
                 status_message.innerHTML = "Building...";
@@ -167,7 +167,7 @@ function run_client(id, ws_address)
             scroll_to_bottom(build_contents);
 
         } else if (result.get_result_type === GetResultType.errors) {
-            console.log("CodeChat client: error output received.");
+            console.log("CodeChat Client: error output received.");
             if (clear_output) {
                 // There was no build output, so just update the errors.
                 build_div.textContent = "";
@@ -187,33 +187,33 @@ function run_client(id, ws_address)
             set_splitter_percent(splitter_size[errors_or_warnings]);
 
         } else if (result.get_result_type === GetResultType.command) {
-            console.log(`CodeChat client: command ${result.text} received.`);
+            console.log(`CodeChat Client: command ${result.text} received.`);
             if (result.text === "shutdown") {
                 // Close this window -- see https://stackoverflow.com/a/54787080. See `close the window`_.
-                outputElement.srcdoc = "<html><body><p>The CodeChat client was shut down. Please close this window.</p></body></html>";
+                outputElement.srcdoc = "<html><body><p>The CodeChat Client was shut down. Please close this window.</p></body></html>";
                 build_div.textContent = "";
                 errors_div.textContent = "";
                 status_message.innerHTML = "Client shut down.";
                 window.open('', '_self').close();
                 // Stop asking for results.
                 ws.close();
-                console.log("CodeChat client: shutdown complete.");
+                console.log("CodeChat Client: shutdown complete.");
             } else if (result.text.startsWith("error: unknown client ")) {
                 // _`Close the window` -- there's no point in asking for more commands. Note: there are some cases where this fails, although I've only seen this once. From the Chrome console, ``Scripts may close only the windows that were opened by them.`` In this case, leave a message asking the user to close the window.
-                outputElement.srcdoc = "<html><body><p>CodeChat client error: lost connection to the CodeChat server. Close this window and restart the editor plug-in.</p></body></html>";
+                outputElement.srcdoc = "<html><body><p>CodeChat Client error: lost connection to the CodeChat Server. Close this window and restart the editor plug-in.</p></body></html>";
                 build_div.textContent = "";
                 errors_div.textContent = "";
                 status_message.innerHTML = "Server disconnected";
                 window.open('', '_self').close();
                 // Stop asking for results.
                 ws.close();
-                console.log(`CodeChat client: shutdown complete due to unknown client; saw ${result.text}.`);
+                console.log(`CodeChat Client: shutdown complete due to unknown client; saw ${result.text}.`);
             } else {
-                console.log(`CodeChat client: Unknown command ${result.text}.`);
+                console.log(`CodeChat Client: Unknown command ${result.text}.`);
             }
 
         } else {
-            console.log(`CodeChat client: Unknown GetResultType: ${result.get_result_type}.`);
+            console.log(`CodeChat Client: Unknown GetResultType: ${result.get_result_type}.`);
         }
     };
 
@@ -230,7 +230,7 @@ function run_client(id, ws_address)
     // The let statement below makes this accessible globally.
     navigate_to_error = function(file_path, line) {
         // TODO.
-        console.log("CodeChat client: TODO", file_path, line);
+        console.log("CodeChat Client: TODO", file_path, line);
     };
 }
 
