@@ -17,9 +17,9 @@
 #   <http://www.gnu.org/licenses/>.
 #
 # *******************************
-# |docname| - The CodeChat server
+# |docname| - The CodeChat Server
 # *******************************
-# The CodeChat server receives requests from the editor/IDE, renders these, then displays them in the CodeChat client; likewise, it listens for CodeChat client requests, processes them, and forwards the results to the editor/IDE.
+# The CodeChat Server receives requests from the editor/IDE, renders these, then displays them in the CodeChat Client; likewise, it listens for CodeChat Client requests, processes them, and forwards the results to the editor/IDE.
 #
 #
 # Imports
@@ -29,7 +29,10 @@
 #
 # Standard library
 # ----------------
+import logging
+import signal
 import socket
+import sys
 import threading
 from typing import Union
 import webbrowser
@@ -52,7 +55,7 @@ from thrift.protocol import TBinaryProtocol
 
 # Local application imports
 # -------------------------
-from . import renderer
+from . import renderer, __version__
 from .gen_py.CodeChat_Services import EditorPlugin
 from .gen_py.CodeChat_Services.ttypes import (
     RenderClientReturn,
@@ -62,7 +65,7 @@ from .gen_py.CodeChat_Services.ttypes import (
 
 # Constants
 # =========
-# The port used for and HTTP connection from the CodeChat Client to the CodeChat Server.
+# The port used for an HTTP connection from the CodeChat Client to the CodeChat Server.
 HTTP_PORT = 5000
 # .. _CodeChat service port:
 #
@@ -71,6 +74,8 @@ THRIFT_PORT = 9090
 
 UNKNOWN_CLIENT = "Unknown client id {}."
 
+logger = logging.getLogger(__name__)
+
 
 # Service provider
 # ================
@@ -87,16 +92,16 @@ class CodeChatHandler:
             ]
         except KeyError:
             location_name = "invalid location"
-        print("get_client({})".format(location_name))
+        logger.info("get_client({})".format(location_name))
         id = self.render_manager.threadsafe_create_client()
         # Get the next ID.
         if id is None:
             ret = RenderClientReturn("", -1, "Duplicate id {}".format(id))
-            print("  => {}".format(ret))
+            logger.info("  => {}".format(ret))
             return ret
         if id < 0:
             ret = RenderClientReturn("", -1, "Server is shutting down.")
-            print("  => {}".format(ret))
+            logger.info("  => {}".format(ret))
             return ret
 
         # Return what's requested.
@@ -125,27 +130,27 @@ class CodeChatHandler:
             ret = RenderClientReturn(
                 "", -1, "Invalid location {}".format(codeChat_client_location)
             )
-            print("  => {}".format(ret))
+            logger.info("  => {}".format(ret))
             return ret
 
         ret = RenderClientReturn(ret_str, id, "")
-        print("  => {}".format(ret))
+        logger.info("  => {}".format(ret))
         return ret
 
     # Render the provided text to HTML, then enqueue it for the web view.
     def start_render(self, text: str, path: str, id: int, is_dirty: bool) -> str:
-        print(
+        logger.info(
             "start_render(path={}, id={}, is_dirty={}, html=\n{}...)\n".format(
                 path, id, is_dirty, text[:80]
             )
         )
         if self.render_manager.threadsafe_start_render(text, path, id, is_dirty):
             # Indicate success.
-            print(" => (empty string)")
+            logger.info(" => (empty string)")
             return ""
         else:
             ret = UNKNOWN_CLIENT.format(id)
-            print(" => {}".format(ret))
+            logger.info(" => {}".format(ret))
             return ret
 
     # _`Shut down an editor client`. The sequence is:
@@ -155,19 +160,19 @@ class CodeChatHandler:
     #
     # This method performs the first step; ``get_result`` performs the second.
     def stop_client(self, id: int) -> str:
-        print("stop_client(id={})".format(id))
+        logger.info("stop_client(id={})".format(id))
         ok = self.render_manager.threadsafe_shutdown_client(id)
         if not ok:
             ret = UNKNOWN_CLIENT.format(id)
-            print("  => {}".format(ret))
+            logger.info("  => {}".format(ret))
             return ret
         # Indicate success.
-        print("  => (empty string)")
+        logger.info("  => (empty string)")
         return ""
 
     # Shut down the server.
     def shutdown_server(self) -> str:
-        print("shutdown_server")
+        logger.info("shutdown_server")
         shutdown_event.set()
         return ""
 
@@ -201,19 +206,19 @@ def editor_plugin_server() -> None:
     server.serve()
 
 
-# Server for the CodeChat client
+# Server for the CodeChat Client
 # ------------------------------
 client_app = Flask(
     __name__,
-    # Serve CodeChat client files statically.
+    # Serve CodeChat Client files statically.
     static_url_path="/static",
     static_folder="CodeChat_Client/static",
-    # Serve the CodeChat client HTML as a template.
+    # Serve the CodeChat Client HTML as a template.
     template_folder="CodeChat_Client/templates",
 )
 
 
-# The endpoint to get the HTML for the CodeChat client.
+# The endpoint to get the HTML for the CodeChat Client.
 @client_app.route("/client")
 def client_html() -> str:
     return render_template(
@@ -250,8 +255,22 @@ def client_data(id: int, url_path: str) -> Union[str, Response]:
 shutdown_event = threading.Event()
 
 
+# Handle signals by setting the shutdown event.
+def signal_handler(signum, frame):
+    shutdown_event.set()
+
+
+# Handle exceptions in the same way.
+def excepthook(type, value, traceback):
+    shutdown_event.set()
+    sys.__excepthook__(type, value, traceback)
+
+
 # Run both servers. This does not (usually) return.
 def run_servers() -> int:
+    print(f"The CodeChat Server, v.{__version__}\n")
+    logging.basicConfig(level=logging.INFO)
+
     # See if the required ports are in use, probably by another instance of this server.
     if (
         is_port_in_use(HTTP_PORT)
@@ -259,10 +278,15 @@ def run_servers() -> int:
         or is_port_in_use(THRIFT_PORT)
     ):
         print(
-            f"Error: ports {HTTP_PORT}, {renderer.WEBSOCKET_PORT}, and/or {THRIFT_PORT} are already in use. Exiting."
+            f"Error: ports {HTTP_PORT}, {renderer.WEBSOCKET_PORT}, and/or {THRIFT_PORT} are already in use.\n"
+            "Hopefully, this means that the CodeChat Server is already running in another process.\n"
+            "Exiting.\n"
         )
         return 1
 
+    # Shut down if any unhandled exception occurs.
+    sys.excepthook = excepthook
+
     # Both servers block when run, so place them in a thread. Mark the Thrift server as a daemon, so it will be killed when the program shuts down.
     editor_plugin_thread = threading.Thread(
         target=editor_plugin_server, name="Editor plugin server", daemon=True
@@ -281,16 +305,17 @@ def run_servers() -> int:
     render_manager_thread = threading.Thread(target=render_manager.run, name="asyncio")
     render_manager_thread.start()
 
+    # On a signal, shut down gracefully.
+    signal.signal(signal.SIGTERM, signal_handler)
+    signal.signal(signal.SIGINT, signal_handler)
+
     # Run the servers in threads until a user-requested shutdown occurs.
-    try:
-        while True:
-            # Ctrl+C is ignored while waiting, so use a 1 second poll.
-            if shutdown_event.wait(1):
-                # If the event is received, exit the loop in order to shut down. First, clear the event so the next invocation of this function will work correctly.
-                shutdown_event.clear()
-                break
-    except KeyboardInterrupt:
-        pass
+    while True:
+        # Ctrl+C is ignored while waiting, so use a 1 second poll.
+        if shutdown_event.wait(1):
+            # If the event is received, exit the loop in order to shut down. First, clear the event so the next invocation of this function will work correctly.
+            shutdown_event.clear()
+            break
 
     print("Shutting down...")
     # This will prevent future editor or web requests from being serviced.
